# Q: Unknown true count
# N: The number of observations to make
# probs: Weighted probabilities (default: uniform)
simulate.unobserved.mass <- function(Q, N, probs = rep(1, Q)) {
  # Necessary when calculating the unobserved mass
  probs <- probs / sum(probs)
  #print(probs)
  
  #observed <- sample.int(Q, N, replace = TRUE, prob = probs)
  observed <- sample(Q, N, replace = TRUE, prob = probs)
  observed.unique <- unique(observed)
  
  observed.counts <- table(observed)
  singletons.indices <- which(observed.counts == 1)
  M1 <- length(singletons.indices)

  doubletons.indices <- which(observed.counts == 2)  
  M2 <- length(doubletons.indices)

  # Robbins, 1968
  estimated.unobserved.mass <- M1 / (N + 1)

  # Because this is a simulation, we can calculate the true mass
  #true.unobserved.mass <- 1 - sum(probs[M1])
  true.unobserved.mass <- 1 - sum(probs[observed.unique])  

  #cat("Estimated unobserved mass =", round(estimated.unobserved.mass, 3), "\n")
  #cat("True unobserved mass      =", round(true.unobserved.mass, 3), "\n")
    
  deviation <- (estimated.unobserved.mass - true.unobserved.mass) / sqrt(true.unobserved.mass)

  return(list(
    deviation = deviation, 
    V = estimated.unobserved.mass, 
    U = true.unobserved.mass, 
    M1 = M1, 
    M2 = M2))
}

#test <- simulate.unobserved.mass(1000, 100)

# probs.type
#   1 = equal
#   2 = uniform
#   3 = normal
#   4 = first 10% is 10 times as likely as the last 90%
create.probs <- function(probs.type, Q) {
  probs.names <- c("equal", "uniform", "normal", "10-90")

  probs <- NULL
  
  if (probs.type == 1) {
    # Uniform
    probs <- rep(1, Q)
  } else if (probs.type == 2) {
    # Random
    probs <- runif(Q)
  } else if (probs.type == 3) {
    # Normal
    # We just take 1000, because the probabilities gets normalized, 
    # and negative values just get a small probability instead. Not optimal, 
    # but hardly a problem.
    probs <- rnorm(Q, 1000, 10)
    probs[which(probs < 0)] <- 10e-6
  } else if (probs.type == 4) {
    # Normal
    # We just take 1000, because the probabilities gets normalized, 
    # and negative values just get a small probability instead. Not optimal, 
    # but hardly a problem.
    if (Q < 10) {
      stop("Q must be greater than or equal to 10 in order to use the 10-90-rule (or else it is equivalent to equal probabilities, so use that if that is what you really wanted).")
    }
    
    first.ten.percent <- floor(0.1*Q)

    # The vector will get normalized
    probs <- c(rep(10, first.ten.percent), rep(1, Q-first.ten.percent))
  } else {
    stop("Unknown probs.type")
  }

  return(list(probs=probs, name=probs.names[probs.type]))
}

# probs.type
#   1 = equal
#   2 = uniform
#   3 = normal
#   4 = first 10% is 10 times as likely as the last 90%

# Q: Unknown true count
# simulations: number of simulations to make for each N
# N's from N.start to N.stop by N.by will be used

analyze.unobserved.mass <- function(probs.type, Q = 1000, simulations = 1000, N.start = 10, N.stop = 1000, N.by = 10) {
  analyze.main <- paste("Deviation in unobserved mass for true population size ", Q, sep="")
  analyze.ylab <- "deviation = (estimated - true) / sqrt(true)"

  probs.object <- create.probs(probs.type, Q)
  probs <- probs.object$probs
  probs.name <- probs.object$name

  Ns <- seq(N.start, N.stop, N.by)
  m <- length(Ns)
  data <- matrix(nrow=simulations, ncol=m)

  # QQ-plot
  j.index.qqplot <- round((m+1) / 2)  

  estimates <- numeric(simulations)
    
  for (j in 1:m) {
    datas <- numeric(simulations)
    for (i in 1:simulations) {
      sim <- simulate.unobserved.mass(Q, Ns[j], probs)
      data[i, j] <- sim$deviation
      
      if (j == j.index.qqplot) {
        estimates[i] <- sim$V
      }
    }
    
    cat(round(100*j / m, 2), "%\n")
  }  

  means <- apply(data, 2, mean)

  pdf(file=paste("figures/unobserved/unobserved-deviation-boxplot-", probs.name, "-", Q, "-", N.start, "-to-", N.stop, "-by-", N.by, ".pdf", sep=""))
  bplot <- boxplot(as.data.frame(data), main = analyze.main, 
    sub = paste("Based on", simulations, "simulations for each N and", probs.name, "probabilities"), 
    xlab="N", ylab=analyze.ylab, names=Ns)
  #points(seq(1, m), means, pch=23, cex=0.75, bg="red")
  #text(seq(1, m), means, labels = formatC(means, format="f", digits=2), pos=3, cex=0.9, col="red")
  points(seq(1, m), means, pch=23, cex=0.75)
  text(seq(1, m), means, labels = formatC(means, format="f", digits=2), pos=3, cex=0.9)
  dev.off()
  
  # QQ-plot 
  probs <- probs / sum(probs)
  N.qq <- Ns[j.index.qqplot]
  
  expected <- sum(probs * (1-probs)^N.qq)
  std <- (estimates - expected) / sqrt(var(estimates))
  
  pdf(file=paste("figures/unobserved/unobserved-deviation-qqplot-", probs.name, "-", Q, "-", N.qq, ".pdf", sep=""))  
  qqnorm(std, main = paste("QQ-plot for N =", N.qq, "observations with true population size", Q),
    sub = paste("Based on", simulations, "simulations and", probs.name, "probabilities"))
  qqline(std)
  dev.off()
  
  return(data)
}

logit <- function(p) {
  return(log(p/(1-p)))
}

logit.inv <- function(x) {
  return(1 / (1 + exp(-x)))
}

# By Maple
logit.diff <- function(p) {
  return(
    (1/(1-p)+p/(1-p)^2)*(1-p)/p
  )
}

# By Maple
logit.diff.diff <- function(p) {
  return(
    (2/(1-p)^2+2*p/(1-p)^3)*(1-p)/p-(1/(1-p)+p/(1-p)^2)*(1-p)/p^2-(1/(1-p)+p/(1-p)^2)/p
  )
}

V.sigmasq.estimate <- function(N, M1, M2) {
  return(M1 / N^2 - (M1 - 2*M2)^2 / N^3)
}

V.confidence.interval <- function(N, V, M1, M2, z.alpha, use.delta = TRUE) {
  sigmasq <- V.sigmasq.estimate(N, M1, M2)

  if (use.delta) {
    EY <- logit(V) + (logit.diff.diff(V) / 2) * sigmasq
    VarY <- (logit.diff(V))^2 * sigmasq
    conf.int <- EY + c(-1, 1) * z.alpha * sqrt(VarY)
    conf.int.trans <- logit.inv(conf.int)
    conf.int.trans <- sort(conf.int.trans)

    if (F) {
      cat("Confidence interval on logit-scale:\n")
      print(conf.int)
      
      cat("Confidence interval on probability-scale:\n")
      print(conf.int.trans)
    }

    return(conf.int.trans)
  } else {
    return(V + c(-1, 1) * z.alpha * sqrt(sigmasq))
  }
}

check.confidence.interval <- function(probs.type, alpha = 0.05, Q = 1000, simulations = 1000, N.start = 10, N.stop = 1000, N.by = 10, plot.with.labels = F) {
  probs.object <- create.probs(probs.type, Q)
  probs <- probs.object$probs
  probs.name <- probs.object$name

  z.alpha <- qnorm(1 - alpha/2)
  
  Ns <- seq(N.start, N.stop, N.by)
  Ns.length <- length(Ns)

  coverages.standard <- numeric(Ns.length)
  coverages.transformed <- numeric(Ns.length)

  widths.standard <- numeric(Ns.length)
  widths.transformed <- numeric(Ns.length)
  widths.max <- 2*z.alpha*sqrt(1 / (Ns + 1))

  coefficents.of.variation <- numeric(Ns.length)
    
  for (j in 1:Ns.length) {       
    coverage.standard <- simulations
    coverage.transformed <- simulations

    width.standard <- 0
    width.transformed <- 0

    coefficents.of.variation.tmp <- numeric(simulations)
    
    for (i in 1:simulations) {      
      N <- Ns[j]
      
      sim <- simulate.unobserved.mass(Q, N, probs)

      # Instead of adding each time U is in the interval, we substract because
      # it hopefully does not happen at the same frequency.

      conf.int.std <- V.confidence.interval(N, sim$V, sim$M1, sim$M2, z.alpha, use.delta = FALSE)
      if (!(conf.int.std[1] <= sim$U && conf.int.std[2] >= sim$U)) {
        coverage.standard <- coverage.standard - 1
      }
            
      conf.int.trans <- V.confidence.interval(N, sim$V, sim$M1, sim$M2, z.alpha, use.delta = TRUE)
      if (!(conf.int.trans[1] <= sim$U && conf.int.trans[2] >= sim$U)) {
        coverage.transformed <- coverage.transformed - 1
      }

      width.standard <- width.standard + (conf.int.std[2] - conf.int.std[1]) / simulations
      width.transformed <- width.transformed + (conf.int.trans[2] - conf.int.trans[1]) / simulations

      if (sim$V != 0) {
        coefficents.of.variation.tmp[i] <- sqrt(V.sigmasq.estimate(N, sim$M1, sim$M2)) / sim$V
      } else {
        coefficents.of.variation.tmp[i] <- 0
      }
    }

    coverages.standard[j] <- coverage.standard / simulations
    coverages.transformed[j] <- coverage.transformed / simulations

    widths.standard[j] <- width.standard
    widths.transformed[j] <- width.transformed

    coefficents.of.variation[j] <- mean(coefficents.of.variation.tmp)
    
    cat(round(100*j / Ns.length, 2), "%\n")    
  }

  ylim.val <- NULL

  if (plot.with.labels) {
    ylim.val <- c(0, 1.1)
  } else {
    ylim.val <- c(0.8, 1)
  }
  
  plot.name <- ifelse(plot.with.labels, "labels", "nolabels")

  pdf(file=paste("figures/unobserved/unobserved-coverages-", plot.name, "-", probs.name, "-", Q, "-", N.start, "-to-", N.stop, "-by-", N.by, ".pdf", sep=""))
  plot(Ns, coverages.standard, 
    ylim = ylim.val, 
    xlab = "N",
    ylab = "Coverage",
    pch = 0,
    main = paste("Confidence interval coverage for true population size Q =", Q),
    sub = paste("Based on ", simulations, " simulations and ", probs.name, " probabilities. Line at ", round(1-alpha, 2), ".", sep=""))
  points(Ns, coverages.transformed, pch=4)
  legend("center", legend = c("Symmetric standard confidence interval", "logit-transformed confidence interval"), pch = c(0, 4))
  abline(a = 1 - alpha, b = 0, lty=2)

  if (plot.with.labels) {
    text(Ns, 1.09, labels = formatC(Ns, format="f", digits=0), pos=3, cex=0.7)

    text(Ns, 1.05, labels = paste("S", formatC(coverages.standard, format="f", digits=3)), pos=3, cex=0.7)
    text(Ns, 1.01, labels = paste("T", formatC(coverages.transformed, format="f", digits=3)), pos=3, cex=0.7)

    text(Ns, 0.19, labels = paste("C", formatC(coefficents.of.variation, format="f", digits=3)), pos=1, cex=0.7)
    #text(Ns, 0.19, labels = paste("M", formatC(widths.max, format="f", digits=3)), pos=1, cex=0.7)    

    text(Ns, 0.15, labels = paste("S", formatC(widths.standard, format="f", digits=3)), pos=1, cex=0.7)
    text(Ns, 0.11, labels = paste("T", formatC(widths.transformed, format="f", digits=3)), pos=1, cex=0.7)
  }
  
  dev.off()

  return(coverages.transformed)
}

