library(xtable)

log.filename <- "log/kernel-smoothing.log"
latex.filename <- "out/kernel-smoothing/kernel-smoothing.tex"

sink(latex.filename)
cat("\\documentclass[a4paper,10pt,oneside,english]{article}\\usepackage{float}\\usepackage{lscape}\\usepackage[pdfstartview=FitH,pdftitle={},pdfauthor={Mikkel Meyer Andersen},pdfsubject={},bookmarksnumbered,pagebackref,bookmarksopen,hyperfootnotes=false]{hyperref}\\urlstyle{same}\\hypersetup{pdfborder=0 0 1}\\input{../../../report/datasetnames}\\begin{document}\\begin{landscape}")
sink()

sink(log.filename)

source("start.R")
source("include-kernel-smoothing.R")
source("include-marginal-logic.R")

results.matrix <- matrix(NA, 
  nrow=length(dataset.names), 
  ncol=2, 
  dimnames=list(paste("¤DATASET:", dataset.names, "¤", sep=""), c("lambda", "Deviance")))

#lambdas.found.matrix <- matrix(NA, 
#  nrow=length(dataset.names), 
#  ncol=1, 
#  dimnames=list(paste("¤DATASET:", dataset.names, "¤", sep=""), c("lambda")))

#deviance.relative.predicted.matrix <- matrix(NA, 
#  nrow=length(dataset.names), 
#  ncol=length(classification.model.names[classification.model.use]), 
#  dimnames=list(paste("¤DATASET:", dataset.names, "¤", sep=""), classification.model.names[classification.model.use]))

for (i in 1:length(dataset.names)) {
#for (i in 2) {
  dataset.name <- dataset.names[i]

  cat("Starting kernel smoothing for dataset \"", dataset.name, "\".\n", sep="")
  
  dataset.compact <- get.compact.dataset(dataset.name)
  dataset.extended <- get.extended.dataset(dataset.name)
  loci.count <- get.loci.count(dataset.name)
  observation.count <- get.observation.count(dataset.name)

  dataset.compact$Freq <- NULL
  N.index <- loci.count + 1

  # Consider using weighted covariance cov.wt instead for gaining performance
  #S <- cov.wt(dataset.compact[, 1:loci.count], wt=dataset.compact$N)$cov
  S <- cov(dataset.extended)
  detS <- det(S)
  Sinv <- qr.solve(S)

  singletons <- get.singletons.count(dataset.name)
  estimated.uncovered <- singletons / (observation.count + 1)

  cat("Estimated uncovered probability mass =", round(estimated.uncovered, 5), "\n")

  # lambda is changed globally because it is used in include-kernel-smoothing.R
  # freq is changed globally to avoid calculating it again after optimal solution is found
  f <- function(l) {
    lambda <<- l

    #Faster than 
    # freq <- apply(dataset.extended 1, g)
    freq <<- dataset.compact$N * apply(dataset.compact[, 1:loci.count], 1, g)
    freq.sum <- sum(freq)
    predicted.uncovered.mass <- 1 - freq.sum

    cat("lambda =", round(lambda, 5), " gives a model assigning uncovered probability mass =", round(predicted.uncovered.mass, 8), "\n")    
    return(estimated.uncovered - predicted.uncovered.mass)
  }

  cat("Trying to find lambda realizing unobserved probability mass.\n")
  #lambda.sol <- uniroot(f, c(0.1, 100))
  lambda.sol <- uniroot(f, c(0.5, 10))
  cat("---------------------------\n")
  cat("Optimal lambda found is lambda =", lambda.sol$root, "\n")
  cat("---------------------------\n")
  lambda <- lambda.sol$root
  results.matrix[i, 1] <- lambda
  
  # Clean up global-mess
  probsEstimated <- freq
  rm(freq)

  # Calculate estimated probabilities under
  results.matrix[i, 2] <- deviance.relfreq.predicted(dataset.compact$N, probsEstimated)

  cat("Done with kernel smoothing for dataset \"", dataset.name, "\".\n\n", sep="")
}

sink() # sink(log.filename)

#tableOutput <- xtable(lambdas.found.matrix,
#      align=c("l", rep("r", ncol(lambdas.found.matrix))),
#      display=c("s", rep("e", ncol(lambdas.found.matrix))),
#      label=paste("tab:kernel-smoothing-lambdas-found", sep=""), 
#      caption=paste("The optimal lambdas found."))
#filename <- paste("out/kernel-smoothing/kernel-smoothing-lambdas-found.tex", sep="")
#print(tableOutput, file=filename)
#do.replacements(filename)

#tableOutput <- xtable(deviance.relative.predicted.matrix,
#      align=c("l", rep("r", ncol(deviance.relative.predicted.matrix))),
#      label=paste("tab:kernel-smoothing-deviance-relative-predicted", sep=""), 
#      caption=paste("Comparing deviance between relative frequencies and predicted probabilities found using kernel smoothing."))
#filename <- paste("out/kernel-smoothing/deviance-relative-predicted.tex", sep="")
#print(tableOutput, file=filename)
#do.replacements(filename)

tableOutput <- xtable(results.matrix,
      align=c("l", "r", "r"),
      display=c("s", "f", "f"),
      label=paste("tab:kernel-smoothing-results", sep=""), 
      caption=paste("Results for the kernel smoothing. The optimal lambdas found is in the lambda-column. The deviance-column is the deviance between relative frequencies and predicted probabilities found using the kernel smoothing."))
filename <- paste("out/kernel-smoothing/kernel-smoothing-results.tex", sep="")
print(tableOutput, file=filename)
do.replacements(filename)

sink(latex.filename, append = TRUE)
#cat("\\input{kernel-smoothing-lambdas-found}\n\\clearpage\n")
#cat("\\input{deviance-relative-predicted}\n\\clearpage\n")
cat("\\input{kernel-smoothing-results}\n\\clearpage\n")
cat("\\end{landscape}\n")
cat("\\end{document}")
sink()

